(function (GCN) {

	'use strict';

	/**
	 * Only grows, never shrinks.
	 * @private
	 * @type {number}
	 */
	var uniqueIdCounter = 0;

	/**
	 * Generates a unique id with an optional prefix.
	 *
	 * The returned value is only unique among other returned values,
	 * not globally.
	 *
	 * @public
	 * @param {string}
	 *        Optional prefix for the id to be generated.
	 * @return {string}
	 *        Never the same string more than once.
	 */
	function uniqueId(prefix) {
		return (prefix || '') + (++uniqueIdCounter);
	}

	function escapePropertyName(name) {
		return name.replace(/\./g, '\\.');
	}

	/**
	 * A regular expression to identify executable script tags.
	 *
	 * @type {RegExp}
	 */
	var rgxpScriptType = /\/(java|ecma)script/i;

	/**
	 * A regular expression to find script tags.
	 *
	 * @type {RegExp}
	 */
	var rgxpScriptTag = new RegExp('<script(\\s[^>]*?)?>', 'ig');

	/**
	 * A regular expression to find script types.
	 *
	 * @type {RegExp}
	 */
	var rgxpType = new RegExp(
		' type\\s*=\\s*'
			+ '[\\"\\\']'
			+ '([^\\"\\\']*[^\\s][^\\"\\\']*)'
			+ '[\\"\\\']',
		'i'
	);

	var rand = Math.random().toString().replace('.', '');

	/**
	 * Places sentinal strings in front of every executable script tag.
	 *
	 * @param {string} html HTML markup
	 * @return {string} HTML string with marked <script> tags.
	 */
	function markScriptTagLocations(html) {
		var i = 0;
		return html.replace(rgxpScriptTag, function (str, substr, offset) {
			var type = substr && substr.match(rgxpType);
			if (!type || rgxpScriptType.test(type)) {
				return rand + (i++) + str;
			}
			return str;
		});
	}

	/**
	 * Replaces the type attribute of <script> tags with a value that will
	 * protect them from being specially handled by jQuery.
	 *
	 * @param {string} html Markup
	 * @return {string} Markup with <script> tags protected from jQuery.
	 */
	function protectScriptTags(html, $scripts) {
		var i;
		var $script;
		var type;
		for (i = 0; i < $scripts.length; i++) {
			$script = $scripts.eq(i).clone();
			type = $script.attr('type');
			if (!type || rgxpScriptType.test(type)) {
				$script.attr('type', rand).attr('data-origtype', type);
				html = html.replace(rand + i, $script[0].outerHTML);
			}
		}
		return html;
	}

	/**
	 * Restores the type attribute for <script> tags that have been processed
	 * via protectScriptTags().
	 *
	 * @param {jQuery.<HTMLElement>} $element Root HTMLElement in which to
	 *                                        restore <script> tags.
	 */
	function restoreScriptTagTypes($element) {
		var $scripts = $element.find('script[type="' + rand + '"]');
		var $script;
		var orig;
		var i;
		for (i = 0; i < $scripts.length; i++) {
			$script = $scripts.eq(i);
			$script.removeClass(rand + i);
			orig = $script.attr('data-origtype');
			if (typeof orig === 'string') {
				$script.attr('type', orig).removeAttr('data-origtype');
			} else {
				$script.removeAttr('type');
			}
		}
	}

	/**
	 * Joins the innerHTML of multiple elements.
	 *
	 * @param {jQuery.<HTMLElement>} $elements
	 */
	function joinContents($elements) {
		var contents = '';
		$elements.each(function () {
			contents += jQuery(this).html();
		});
		return contents;
	}

	/**
	 * Inserts the inner HTML of the given HTML markup while preserving
	 * <script> tags, and still allowing jQuery to execute them.
	 *
	 * @param {jQuery.<HTMLElement>} $element
	 * @param {string} html
	 */
	function insertInnerHTMLWithScriptTags($element, html) {
		if (!rgxpScriptTag.test(html)) {
			$element.html(jQuery(html).contents());
			return;
		}
		var marked = markScriptTagLocations(html);
		var $scripts = jQuery(html).filter('script');
		var $html = jQuery(marked);
		var contents = joinContents($html.filter(':not(script)'));
		contents = protectScriptTags(contents, $scripts);
		$element.html(contents).append($scripts);
		restoreScriptTagTypes($element);
	}

	GCN.uniqueId = uniqueId;
	GCN.escapePropertyName = escapePropertyName;
	GCN.insertInnerHTMLWithScriptTags = insertInnerHTMLWithScriptTags;

}(GCN));
